==> notify_widget-1.0.0.tar.gz: notify_widget/src/NotifyWidgetApi.php <==
<?php declare(strict_types = 1);

namespace Drupal\notify_widget;

use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountProxyInterface;

/**
 * Provides an API for sending notifications to users.
 */
final class NotifyWidgetApi {

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The config object for the notify widget.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The cache tag invalidator service.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $cacheTagsInvalidator;

  /**
   * Constructs a NotifyWidgetApi object.
   */
  public function __construct(
    Connection $database,
    ConfigFactoryInterface $configFactory,
    AccountProxyInterface $currentUser,
    CacheTagsInvalidatorInterface $cacheTagsInvalidator
  ) {
    $this->database = $database;
    $this->config = $configFactory->get('notify_widget.settings');
    $this->currentUser = $currentUser;
    $this->cacheTagsInvalidator = $cacheTagsInvalidator;
  }

  /**
   * Send a notification to a user.
   *
   * @param string $source
   *   The notification source.
   * @param string $type
   *   The notification type.
   * @param string $title
   *   The notification title.
   * @param string $text
   *   The notification text.
   * @param int $uid
   *   The recipient user ID.
   * @param string $link
   *   The link the notification should take the user to.
   *
   * @throws \Exception
   */
  public function send(
    string $source,
    string $type,
    string $title,
    string $text,
    int $uid = 0,
    string $link = ''
  ): void {
    // If no user ID has been specified, assume the current user ID.
    if ($uid === 0) {
      $uid = $this->currentUser->id();
    }

    // Insert the details into the notify_widget database table.
    $this->database->insert('notify_widget')
      ->fields([
        'uid' => $uid,
        'source' => $source,
        'read' => 0,
        'notification_type' => $type,
        'notification_title' => $title,
        'notification_text' => $text,
        'link' => $link,
      ])
      ->execute();

    // Invalidate the notify widget cache for the current user.
    $this->clearNotifyWidgetCacheForUser();
  }

  /**
   * Mark a notification as read.
   *
   * @param int $id
   *   The notification ID to set as read.
   */
  public function markAsRead(int $id): void  {
    $this->setNotificationStatus($id, 1);
  }

  /**
   * Mark a notification as unread.
   *
   * @param int $id
   *   The notification ID to set as unread.
   */
  public function markAsUnread(int $id): void  {
    $this->setNotificationStatus($id, 0);
  }

  /**
   * Set notification status.
   *
   * @param int $id
   *   The ID of the notification.
   * @param int $status
   *   The status to set the notification to. 0 = unread, 1 = read.
   */
  private function setNotificationStatus(int $id, int $status): void {
    $this->database->update('notify_widget')
      ->fields([
        'read' => $status,
      ])
      ->condition('id', $id)
      ->execute();

    // Invalidate the notify widget cache for the current user.
    $this->clearNotifyWidgetCacheForUser();
  }

  /**
   * Get the notifications for a user.
   *
   * @param int $id
   *   ID of a specific notification to fetch. All are returned for
   *   the curren t user of no ID is supplied.
   *
   * @return array
   *   An array of stdClass objects, where each object is
   *   a DB row (notification).
   */
  public function getNotificationsForUser(int $id = 0): array {
    // Get the maximum number of notifications to show in the widget.
    $maxNotifications = $this->config->get('max_notifications') ?? 10;

    // Get the notification details for the user.
    $query = $this->database->select('notify_widget', 'n')
      ->fields('n', [])
      ->condition('uid', $this->currentUser->id())
      ->orderBy('timestamp', 'desc')
      ->range(0, $maxNotifications);

    if ($id <> 0) {
      $query->condition('id', $id);
    }

    return $query->execute()->fetchAll();
  }

  /**
   * Get the number of unread notifications for a user.
   *
   * @return int
   *   The number of unread notifications.
   */
  public function getUnreadNotificationsCount(): int {
    // Get a count of the total number of unread notifications.
    $unreadCountQuery = $this->database->select('notify_widget', 'n')
      ->fields('n', ['id'])
      ->condition('uid', $this->currentUser->id())
      ->condition('read', 0);
    return (int) $unreadCountQuery->countQuery()->execute()->fetchField() ?? 0;
  }

  /**
   * Clear the notify widget cache for the logged in user.
   */
  public function clearNotifyWidgetCacheForUser(): void {
    // Invalidate the notify widget cache for the current user.
    $this->cacheTagsInvalidator->invalidateTags(['notify_widget:' . $this->currentUser->id()]);
  }

}

==> notify_widget-1.0.3.tar.gz: notify_widget/src/NotifyWidgetApi.php <==
<?php declare(strict_types = 1);

namespace Drupal\notify_widget;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountProxyInterface;

/**
 * Provides an API for sending notifications to users.
 */
final class NotifyWidgetApi {

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The config object for the notify widget.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The cache tag invalidator service.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $cacheTagsInvalidator;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs a NotifyWidgetApi object.
   */
  public function __construct(
    Connection $database,
    ConfigFactoryInterface $configFactory,
    AccountProxyInterface $currentUser,
    CacheTagsInvalidatorInterface $cacheTagsInvalidator,
    TimeInterface $time
  ) {
    $this->database = $database;
    $this->config = $configFactory->get('notify_widget.settings');
    $this->currentUser = $currentUser;
    $this->cacheTagsInvalidator = $cacheTagsInvalidator;
    $this->time = $time;
  }

  /**
   * Send a notification to a user.
   *
   * @param string $source
   *   The notification source.
   * @param string $type
   *   The notification type.
   * @param string $title
   *   The notification title.
   * @param string $text
   *   The notification text.
   * @param int $uid
   *   The recipient user ID.
   * @param string $link
   *   The link the notification should take the user to.
   *
   * @throws \Exception
   */
  public function send(
    string $source,
    string $type,
    string $title,
    string $text,
    int $uid = 0,
    string $link = ''
  ): void {
    // If no user ID has been specified, assume the current user ID.
    if ($uid === 0) {
      $uid = $this->currentUser->id();
    }

    // Insert the details into the notify_widget database table.
    $this->database->insert('notify_widget')
      ->fields([
        'uid' => $uid,
        'source' => $source,
        'read' => 0,
        'notification_type' => $type,
        'notification_title' => $title,
        'notification_text' => $text,
        'link' => $link,
        'timestamp' => $this->time->getRequestTime(),
      ])
      ->execute();

    // Invalidate the notify widget cache for the current user.
    $this->clearNotifyWidgetCacheForUser();
  }

  /**
   * Mark a notification as read.
   *
   * @param int $id
   *   The notification ID to set as read.
   */
  public function markAsRead(int $id): void  {
    $this->setNotificationStatus($id, 1);
  }

  /**
   * Mark a notification as unread.
   *
   * @param int $id
   *   The notification ID to set as unread.
   */
  public function markAsUnread(int $id): void  {
    $this->setNotificationStatus($id, 0);
  }

  /**
   * Set notification status.
   *
   * @param int $id
   *   The ID of the notification.
   * @param int $status
   *   The status to set the notification to. 0 = unread, 1 = read.
   */
  private function setNotificationStatus(int $id, int $status): void {
    $this->database->update('notify_widget')
      ->fields([
        'read' => $status,
      ])
      ->condition('id', $id)
      ->execute();

    // Invalidate the notify widget cache for the current user.
    $this->clearNotifyWidgetCacheForUser();
  }

  /**
   * Get the notifications for a user.
   *
   * @param int $id
   *   ID of a specific notification to fetch. All are returned for
   *   the curren t user of no ID is supplied.
   *
   * @return array
   *   An array of stdClass objects, where each object is
   *   a DB row (notification).
   */
  public function getNotificationsForUser(int $id = 0): array {
    // Get the maximum number of notifications to show in the widget.
    $maxNotifications = $this->config->get('max_notifications') ?? 10;

    // Get the notification details for the user.
    $query = $this->database->select('notify_widget', 'n')
      ->fields('n', [])
      ->condition('uid', $this->currentUser->id())
      ->orderBy('timestamp', 'desc')
      ->range(0, $maxNotifications);

    if ($id <> 0) {
      $query->condition('id', $id);
    }

    return $query->execute()->fetchAll();
  }

  /**
   * Get the number of unread notifications for a user.
   *
   * @return int
   *   The number of unread notifications.
   */
  public function getUnreadNotificationsCount(): int {
    // Get a count of the total number of unread notifications.
    $unreadCountQuery = $this->database->select('notify_widget', 'n')
      ->fields('n', ['id'])
      ->condition('uid', $this->currentUser->id())
      ->condition('read', 0);
    return (int) $unreadCountQuery->countQuery()->execute()->fetchField() ?? 0;
  }

  /**
   * Clear the notify widget cache for the logged in user.
   */
  public function clearNotifyWidgetCacheForUser(): void {
    // Invalidate the notify widget cache for the current user.
    $this->cacheTagsInvalidator->invalidateTags(['notify_widget:' . $this->currentUser->id()]);
  }

}
